#! python
#
# This is a thin wrapper to load the rfc2271 implementation.
#
# This file is part of pySerial. https://github.com/pyserial/pyserial
# (C) 2011 Chris Liechti <cliechti@gmx.net>
#
# SPDX-License-Identifier:    BSD-3-Clause

from serial.rfc2217 import Serial  # noqa
